
GuildDelta_memberdata = {}

local chatframe
local function setchatframe()
    GuildDelta_chatframe = GuildDelta_chatframe or 2
    if type(GuildDelta_chatframe) == "number" then
        chatframe = getglobal("ChatFrame"..GuildDelta_chatframe)
    else
        chatframe = getglobal(GuildDelta_chatframe) --[[fallback for safety]] or ChatFrame2
    end
end

local function prt (...)
    return chatframe:AddMessage(...)
end
local function cprt (...)
    return prt(..., 255/255, 26/255, 160/255)
end

local function GuildDelta_PrintHelp()
    for _,x in pairs{
        "GuildDelta 3.2.2 Usage:  '/guilddelta X', where X is one of:",
        "help      this message",
        "reset     restores default output setting",
        "print     prints each chat window's number in that window for reference",
        "<num>     a number from 1 through 7, future output will go to chat window <num>",
        "<name>    (advanced) the name of a chat frame, future output will go to it"
    } do DEFAULT_CHAT_FRAME:AddMessage(x) end
end


local current_guild_info = function (N)
    local ret = {}
    for i = 1, N do
        local name,rank,_,_,_,_,pnote,onote = GetGuildRosterInfo(i)
        if name then -- redundant, but apparently happens on extreme lag
            table.insert(ret,
                {name   = name,
                 rank   = rank,   -- as string, not as index
                 --level  = x[4], -- NOT CURRENTLY USED
                 pnote  = pnote,  -- public
                 onote  = onote,  -- officer
                })
        end
    end
    table.sort(ret, function (l,r) return l.name < r.name end)
    return ret
end

AutoCompleteInfoDelayer:HookScript("OnFinished", function() GuildDelta:RegisterEvent("GUILD_ROSTER_UPDATE") end)

function GuildDelta_OnEvent (frame, event)
    if event == "PLAYER_LOGIN" then
        if IsInGuild() then
            SlashCmdList["GUILDDELTA"] = function (txt)
                txt = strtrim(txt)
                if txt == "print" then
                    for i = 1, 7 do
                        local f = getglobal ("ChatFrame"..i)
                        if f then
                            f:AddMessage ("This is chat window "..i..".",1,1,1)
                        end
                    end

                elseif txt == "help" or txt == "?" or txt == "" then
                    GuildDelta_PrintHelp()

                elseif txt == "reset" then
                    return SlashCmdList["GUILDDELTA"]("2")

                else
                    -- "/guilddelta N"
                    local n = tonumber (txt or "")
                    if not n then
                        -- string
                        if getglobal(txt) then
                            -- Could check that it refers to a message frame, but enh.
                            GuildDelta_chatframe = txt
                        else
                            DEFAULT_CHAT_FRAME:AddMessage ("'"..txt.."' is not the name of an existing frame.")
                            return
                        end
                    elseif (n < 1) or (n > 7) then
                        -- if they type something like "/guilddelta 3.141" then screw 'em
                        GuildDelta_PrintHelp()
                        return
                    else
                        -- good ChatFrame number
                        GuildDelta_chatframe = n
                    end
                    DEFAULT_CHAT_FRAME:AddMessage ("GuildDelta will now print to chat window "
                                                   ..GuildDelta_chatframe..".")
                    setchatframe()
                end
            end
            SLASH_GUILDDELTA1 = "/guilddelta"
            setchatframe()
        else
            DEFAULT_CHAT_FRAME:AddMessage ("You are not in a guild; GuildDelta not loading.")
        end

    elseif event == "GUILD_ROSTER_UPDATE" then
        local current_n = GetNumGuildMembers(true)
        if current_n <= 0 then
            -- catch the hell up, servers...
            GuildRoster(); return;
        end
        frame:UnregisterEvent("GUILD_ROSTER_UPDATE")

        local guild, realm = (GetGuildInfo("player")), GetRealmName()
        if GuildDelta_memberdata[realm] 
           and GuildDelta_memberdata[realm][guild]
           and #(GuildDelta_memberdata[realm][guild]) > 0
           --and #(GuildDelta_memberdata[realm][guild][1]) = 4... grr.
        then
            -- table.insert with notes if available
            local function tins (t, x)
                local s = x.name
                if x.onote and (x.onote ~= "") then
                    s = s .. "(O: " .. x.onote .. ")"
                end
                if x.pnote and (x.pnote ~= "") then
                    s = s .. "(" .. x.pnote .. ")"
                end
                table.insert(t, s)
            end

            -- build the current list
            local previous, current = GuildDelta_memberdata[realm][guild],
                                      current_guild_info(current_n)
            local previous_n = #previous

            -- walk both and do equivalence comparison
            local joined, left, rank, notes = {}, {}, {}, {}
            local p, c = 1, 1
            while p <= previous_n and c <= current_n do
                local P, C = previous[p], current[c]   -- cache for lazy typing :-)

                if P.name == C.name then
                    -- normal case
                    p = p + 1
                    c = c + 1
                    -- but can now compare details
                    if C.rank and (P.rank ~= C.rank) then
                        table.insert(rank, {C.name, P.rank.." --> "..C.rank})
                    end
                    if C.pnote and (P.pnote ~= C.pnote) then
                        table.insert(notes, {C.name, C.pnote})
                    end
                    if C.onote and (P.onote ~= C.onote) then
                        table.insert(notes, {C.name, "[O]: "..C.onote})
                    end

                elseif P.name < C.name then
                    -- entry at index p not at c -> somebody has left
                    tins (left, P)
                    p = p + 1

                else
                    -- entry at index c not at p -> somebody has joined
                    tins (joined, C)
                    c = c + 1
                end
            end

            -- leftovers
            for i = p, previous_n do
                tins (left, previous[i])
            end
            for i = c, current_n do
                tins (joined, current[i])
            end

            -- show results
            local m
            if #left > 0 then
                m = "Players who have left the guild:  "
                m = m .. table.concat(left, ", ")
                cprt(m)
            end

            if #joined > 0 then
                m = "Players who have joined the guild:  "
                m = m .. table.concat(joined, ", ")
                cprt(m)
            end

            if #rank > 0 then
                cprt("Players whose rank has changed:")
                for i = 1, #rank do
                    cprt(rank[i][1]..":  "..rank[i][2])
                end
            end

            if #notes > 0 then
                cprt("Players whose notes have changed:")
                for i = 1, #notes do
                    cprt(notes[i][1]..": \""..notes[i][2].."\"")
                end
            end

            GuildDelta_memberdata[realm][guild] = current

        else
            -- new user, or new guild, or ...
            prt("GuildDelta initializing roster...")
            GuildDelta_memberdata[realm] = GuildDelta_memberdata[realm] or {}
            GuildDelta_memberdata[realm][guild] = current_guild_info(current_n)
        end
    end
end

